{% extends "base.html" %}
{% load i18n %}

{% block content %}

<h1>{% translate "Preview your submission" %}</h1>

<table>
{% for field in form %}
<tr>
<th>{{ field.label }}:</th>
<td>{{ field.data }}</td>
</tr>
{% endfor %}
</table>

<p>{% blocktranslate %}Security hash: {{ hash_value }}{% endblocktranslate %}</p>

<form action="" method="post">{% csrf_token %}
{% for field in form %}{{ field.as_hidden }}
{% endfor %}
<input type="hidden" name="{{ stage_field }}" value="2" />
<input type="hidden" name="{{ hash_field }}" value="{{ hash_value }}" />
<p><input type="submit" value="{% translate "Submit" %}" /></p>
</form>

<h1>{% translate "Or edit it again" %}</h1>

<form action="" method="post">{% csrf_token %}
<table>
{{ form.as_table }}
</table>
<input type="hidden" name="{{ stage_field }}" value="1" />
<p><input type="submit" value="{% translate "Preview" %}" /></p>
</form>

{% endblock %}
