{% extends "base.html" %}
{% load i18n %}

{% block content %}

{% if form.errors %}<h1>{% translate "Please correct the following errors" %}</h1>
{% else %}<h1>{% translate "Submit" %}</h1>{% endif %}

<form action="" method="post">{% csrf_token %}
<table>
{{ form.as_table }}
</table>
<input type="hidden" name="{{ stage_field }}" value="1" />
<p><input type="submit" value="{% translate "Preview" %}" /></p>
</form>

{% endblock %}
